//
//  L7SErrors.h
//  
//
//  Created by Anthony Yu on 13-07-05.
//
//

#import <Foundation/NSObject.h>


/**
 There can be various errors and exceptions occurring in SDK at runtime, apps can subscribe for error callback by implementing an optional method in `DidReceiveErrors:` of `L7SClientProtocol`, and then set the delegate to `L7SClientManager`, for example:
 
    [[L7SClientManager sharedClientManager] setDelegate:self];
 
 The errors are encapsulated in an NSError object.  We categorize the SDK system error is three domains.  They are defined with the following constants:
 
    extern NSString * const L7SAuthenticationDomain;
    extern NSString * const L7SRegistrationDomain;
    extern NSString * const L7SLocationDomain;
    extern NSString * const L7SNetworkDomain;
    extern NSString * const L7SJWTValidationDomain;
    extern NSString * const L7SEnterpriseBrowserDomain;
    extern NSString * const L7SClientCredentialDomain;
 
 The following is a set of error codes defined:
 
    //Error code for authentication domain
    L7SAuthenticationLoginError = -101, //encountering login error
    L7SAuthenticationJWTError = -102,  //encountering error when exchange credentials with JWT
    L7SAuthenticationRefreshError = -103,  //encountering error when refresh credenitcails
    L7SAuthenticationLogoutError = -104,  //encountering logout error
    L7SAuthenticationLogoffError = -105,  //encountering logoff error
    L7SSocialLoginError = -106,  //encountering social login error
    L7SQRCodeAuthenticationError = -107,  // encoutering an error when the SDK fails to retrieve a session after its QR code is scanned.
    L7SQRCodeAuthorizationError = -108,   // encoutering an error when the SDK fails to authorize with a QR code
 
 
    //Error code for registration domain
    L7SRegistrationError = -201,  //encountering errors during device registration
    L7SDeRegistrationError = -202, //encountering errors during device de-registration
 
    //Error code for location domain
    L7SLocationError = -301,  //encountering location service error
    L7SLocationUnauthorizedError = -302, //location is unauthorized

    //Error code for network
    L7SNetworkError = -401,  //encountering network service error
 
    //HTTPCall error
    L7SHTTPCallError = -501,  //only used in HTTP call error through PhoneGap plugin
 
    //JWT validation errors
    L7SJWTSignatureNotMatch = -701,
    L7SJWTAUDNotMatch = -702,
    L7SJWTAZPNotMatch = -703,
    L7SJWTExpired = -704,
    L7SJWTInvalidFormat = -705,
 
    //Enterprise browser error
    L7SEnterpriseBrowserInvalidJSON = -801,
    L7SEnterrpiseBrowserNativeAppNotExist = -802,
    L7SEnterrpiseBrowserInvalidURLError = -803,
    L7SEnterrpiseBrowserAppNotExist = -804
 
    //Dynamic client credential error
    L7SDynamicClientCredentialError = -901
 
 Here is an example of responding to an error callback:
    
    - (void)DidReceiveError:(NSError *)error{
        switch([error code]) {
            case L7SAuthenticationJWTError:
            {
                UIAlertView *alert = [[UIAlertView alloc] initWithTitle:[[error userInfo] objectForKey:NSLocalizedFailureReasonErrorKey]
                                                                message:[[error userInfo] objectForKey:NSLocalizedDescriptionKey]
                                                               delegate:nil
                                                      cancelButtonTitle:@"OK"
                                                      otherButtonTitles:nil];
                [alert show];
                break;
            }
            default:
            break:
        }
    }

 
 */



typedef enum {
    
    //Error code for authentication domain
    L7SAuthenticationLoginError = -101,
    L7SAuthenticationJWTError = -102,
    L7SAuthenticationRefreshError = -103,
    L7SAuthenticationLogoutError = -104,
    L7SAuthenticationLogoffError = -105,
    L7SSocialLoginError = -106,
    L7SQRCodeAuthenticationError = -107,
    L7SQRCodeAuthorizationError = -108,
    
    //Error code for registration domain
    L7SRegistrationError = -201,
    L7SDeRegistrationError = -202,
    
    //Error code for location domain
    L7SLocationError = -301,
    L7SLocationUnauthorizedError = -302,
    
    //Error code for network
    L7SNetworkError = -401,
    
    //HTTPCall error
    L7SHTTPCallError = -501,
    
    //JWT validation errors
    L7SJWTSignatureNotMatch = -701,
    L7SJWTAUDNotMatch = -702,
    L7SJWTAZPNotMatch = -703,
    L7SJWTExpired = -704,
    L7SJWTInvalidFormat = -705,
    
    //Enterprise browser error
    L7SEnterpriseBrowserInvalidJSON = -801,
    L7SEnterrpiseBrowserNativeAppNotExist = -802,
    L7SEnterrpiseBrowserInvalidURLError = -803,
    L7SEnterrpiseBrowserAppNotExist = -804,
    
    //Dynamic client credential error
    L7SDynamicClientCredentialError = -901

    
} L7SError;


extern NSString * const L7SAuthenticationDomain;
extern NSString * const L7SRegistrationDomain;
extern NSString * const L7SLocationDomain;
extern NSString * const L7SNetworkDomain;
extern NSString * const L7SJWTValidationDomain;
extern NSString * const L7SEnterpriseBrowserDomain;
extern NSString * const L7SClientCredentialDomain;


//Authentication Error
extern NSString* const AuthenticationError;
extern NSString* const InvalidUsernamePasswordErrorMessage;
extern NSString* const InvaidJWTErrorMessage;
extern NSString* const InvaidRefreshTokenErrorMessage;
extern NSString* const SocialLoginErrorMessage;
extern NSString* const EnterpriseLoginDisabledErrorMessage;
extern NSString* const QRCodeAuthenticationErrorMessage;
extern NSString* const QRCodeAuthrorizeErrorMessage;

//Logoff Error
extern NSString* const LogoffError;
extern NSString* const LogoffErrorMessage;
extern NSString* const LogoffErrorResponseMessage;

//Logout Error
extern NSString* const LogoutError;
extern NSString* const LogoutErrorMessage;
extern NSString* const LogoutErrorResponseMessage;

//Location Error
extern NSString* const LocationError;
extern NSString* const LocationUnauthorizedError;
extern NSString* const LocationErrorMessage;
extern NSString* const LocationErrorUnauthorizedMessage;


//Registration Error
extern NSString* const RegistrationError;
extern NSString* const RegistrationErrorResponseMessage;


//De-Registration Error
extern NSString* const DeRegistrationError;
extern NSString* const DeRegistrationErrorMessage;
extern NSString* const DeRegistrationErrorResponseMessage;


//Network Error
extern NSString* const NetworkError;
extern NSString* const NetworkUnavailableMessage;

//JWT validation errors
extern NSString* const JWTValidationError;
extern NSString* const JWTSignatureNotMatchErrorMessage; //-701
extern NSString* const JWTAUDNotMatchErrorMessage; //-702
extern NSString* const JWTAZPNotMatchErrorMessage; //-703
extern NSString* const JWTExpiredErrorMessage; //-704
extern NSString* const JWTInvalidFormatErrorMessage; //-705

//Eneterprise Browser Error
extern NSString* const EnterpriseBrowserError;
extern NSString* const EnterpriseBrowserInvalidJSONMessage;
extern NSString* const EnterrpiseBrowserNativeAppNotExistMessage;
extern NSString* const EnterrpiseBrowserAppNotExistMessage;
extern NSString* const EnterrpiseBrowserInvalidURLErrorMessage;

//Client credential error
extern NSString* const ClientCredentialError;
extern NSString* const DynamicClientCredentialErrorMessage;

@interface L7SErrors : NSObject 


+ (void) errorWithError:(NSError *) error domain:(NSString *) domain code:(L7SError ) errorCode reason:(NSString *) reason description:(NSString *) description;

+ (void) errorWithUserInfo:(NSMutableDictionary *) userInfo domain:(NSString *) domain code:(L7SError ) errorCode reason:(NSString *) reason description:(NSString *) description;

@end
